@@ -11,6 +11,8 @@
 #include <Eris/Response.h>
 #include <Eris/DeleteLater.h>
 #include <Eris/Timeout.h>
+#include "SpawnPoint.h"
+#include "CharacterType.h"
 
 #include <Atlas/Objects/Entity.h>
 #include <Atlas/Objects/Operation.h>
@@ -18,8 +20,12 @@
 
 #include <algorithm>
 #include <cassert>
+#include <iostream>
+
+#pragma warning(disable: 4068)  //unknown pragma
 
 using Atlas::Objects::Root;
+using Atlas::Message::Element;
 using namespace Atlas::Objects::Operation;
 using Atlas::Objects::Entity::RootEntity;
 using Atlas::Objects::Entity::Anonymous;
@@ -46,7 +52,7 @@ public:
     {
         // logout
         if (op->getClassNo() == LOGOUT_NO) {
-            debug() << "Account reciev forced logout from server";
+            debug() << "Account received forced logout from server";
             m_account->internalLogout(false);
             return HANDLED;
         }
@@ -273,6 +279,34 @@ void Account::createCharacterHandler(long serialno)
 }
 */
 
+Result Account::takeTransferredCharacter(const std::string &id, const std::string &key)
+{
+    if (!m_con->isConnected()) return NOT_CONNECTED;
+    if (m_status != LOGGED_IN) {
+        if ((m_status == CREATING_CHAR) || (m_status == TAKING_CHAR)) {
+            error() << "duplicate char creation / take";
+            return DUPLICATE_CHAR_ACTIVE;
+        } else {
+            error() << "called createCharacter on unconnected Account, ignoring";
+            return NOT_LOGGED_IN;
+        }
+    }
+
+    Anonymous what;
+    what->setId(id);
+    what->setAttr("possess_key", key);
+
+    Look l;
+    l->setFrom(getId());
+    l->setArgs1(what);
+    l->setSerialno(getNewSerialno());
+    m_con->send(l);
+
+    m_con->getResponder()->await(l->getSerialno(), this, &Account::avatarResponse);
+    m_status = TAKING_CHAR;
+    return NO_ERR;
+}
+
 Result Account::takeCharacter(const std::string &id)
 {
     if (m_characterIds.count(id) == 0) {
@@ -384,11 +418,20 @@ void Account::loginResponse(const RootOperation& op)
 
 void Account::loginComplete(const AtlasAccount &p)
 {
-    if (m_status != LOGGING_IN)
+    if (m_status != LOGGING_IN) {
         error() << "got loginComplete, but not currently logging in!";
+    }
 
-    if (p->getUsername()  != m_username)
-        error() << "missing or incorrect username on login INFO";
+    if (!p.isValid()) {
+        error() << "no account in response.";
+        return;
+    }
+
+    //The user name being different should not be a fatal thing.
+    if (p->getUsername() != m_username) {
+        warning() << "received username does not match existing";
+        m_username = p->getUsername();
+    }
 
     m_status = LOGGED_IN;
     m_accountId = p->getId();
@@ -403,13 +446,20 @@ void Account::loginComplete(const AtlasAccount &p)
     m_timeout.reset();
 }
 
+void Account::avatarLogoutRequested(Avatar* avatar)
+{
+    AvatarDeactivated.emit(avatar);
+    delete avatar;
+}
+
 void Account::updateFromObject(const AtlasAccount &p)
 {
     m_characterIds = StringSet(p->getCharacters().begin(), p->getCharacters().end());
+    m_parents = p->getParents();
 
     if(p->hasAttr("character_types") == true)
     {
-        Atlas::Message::Element CharacterTypes(p->getAttr("character_types"));
+        const Atlas::Message::Element& CharacterTypes(p->getAttr("character_types"));
 
         if(CharacterTypes.isList() == true)
         {
@@ -436,6 +486,53 @@ void Account::updateFromObject(const AtlasAccount &p)
             error() << "Account has attribute \"character_types\" which is not of type List.";
         }
     }
+
+    if (p->hasAttr("spawns")) {
+        const Atlas::Message::Element& spawns(p->getAttr("spawns"));
+
+        if (spawns.isList()) {
+            m_spawnPoints.clear();
+            const Atlas::Message::ListType & spawnsList(spawns.asList());
+            for (Atlas::Message::ListType::const_iterator I = spawnsList.begin(); I != spawnsList.end(); ++I) {
+                if (I->isMap()) {
+                    const Atlas::Message::MapType& spawnMap = I->asMap();
+                    Atlas::Message::MapType::const_iterator spawnNameI = spawnMap.find("name");
+                    if (spawnNameI != spawnMap.end()) {
+                        const Atlas::Message::Element& name(spawnNameI->second);
+                        if (name.isString()) {
+                            CharacterTypeStore characterTypes;
+                            Atlas::Message::MapType::const_iterator characterTypesI = spawnMap.find("character_types");
+
+                            if (characterTypesI != spawnMap.end()) {
+                                const Atlas::Message::Element& characterTypesElement(characterTypesI->second);
+                                if (characterTypesElement.isList()) {
+                                    const Atlas::Message::ListType & characterTypesList(characterTypesElement.asList());
+                                    for (Atlas::Message::ListType::const_iterator J = characterTypesList.begin(); J != characterTypesList.end(); ++J)
+                                    {
+                                        if (J->isString()) {
+                                            characterTypes.push_back(CharacterType(J->asString(), ""));
+                                        } else {
+                                            error() << "Character type is not of type string.";
+                                        }
+                                    }
+                                } else {
+                                    error() << "Character type element is not of type list.";
+                                }
+                            }
+                            SpawnPoint spawnPoint(name.asString(), characterTypes, "");
+                            m_spawnPoints.insert(SpawnPointMap::value_type(spawnPoint.getName(), spawnPoint));
+                        } else {
+                            error() << "Spawn name is not a string.";
+                        }
+                    } else {
+                        error() << "Spawn point has no name defined.";
+                    }
+                }
+            }
+        } else {
+            error() << "Account has attribute \"spawns\" which is not of type List.";
+        }
+    }
 }
 
 std::string getErrorMessage(const RootOperation & err)
@@ -508,7 +605,7 @@ void Account::avatarResponse(const RootOperation& op)
             return;
         }
 
-        Avatar* av = new Avatar(this, ent->getId());
+        Avatar* av = new Avatar(*this, ent->getId());
         AvatarSuccess.emit(av);
         m_status = Account::LOGGED_IN;
 
@@ -632,7 +729,7 @@ void Account::avatarLogoutResponse(const RootOperation& op)
 
     ActiveCharacterMap::iterator it = m_activeCharacters.find(charId);
     if (it == m_activeCharacters.end()) {
-        warning() << "character ID " << charId << " does not crrespond to an active avatar.";
+        warning() << "character ID " << charId << " does not correspond to an active avatar.";
         return;
     }
 
