@@ -22,9 +22,14 @@
 #include <Atlas/Objects/Entity.h>
 #include <Atlas/Objects/Anonymous.h>
 
+#pragma warning(disable: 4068)  //unknown pragma
+
 using namespace Atlas::Objects::Operation;
 using Atlas::Objects::Root;
 using Atlas::Objects::Entity::Anonymous;
+using WFMath::CoordType;
+using WFMath::TimeStamp;
+using WFMath::numeric_constants;
 using WFMath::TimeStamp;
 using namespace Atlas::Message;
 using Atlas::Objects::smart_dynamic_cast;
@@ -32,12 +37,13 @@ using Atlas::Objects::smart_dynamic_cast;
 namespace Eris
 {
 
-Avatar::Avatar(Account* pl, const std::string& entId) :
+Avatar::Avatar(Account& pl, const std::string& entId) :
     m_account(pl),
     m_entityId(entId),
     m_entity(NULL),
     m_stampAtLastOp(TimeStamp::now()),
-    m_lastOpTime(0.0)
+    m_lastOpTime(0.0),
+    m_isAdmin(false)
 {
     m_view = new View(this);
     m_entityAppearanceCon = m_view->Appearance.connect(sigc::mem_fun(this, &Avatar::onEntityAppear));
@@ -50,7 +56,7 @@ Avatar::Avatar(Account* pl, const std::string& entId) :
 
 Avatar::~Avatar()
 {
-    m_account->internalDeactivateCharacter(this);
+    m_account.internalDeactivateCharacter(this);
 
     delete m_router;
     delete m_view;
@@ -70,7 +76,7 @@ void Avatar::deactivate()
 
 #pragma mark -
 
-void Avatar::drop(Entity* e, const WFMath::Point<3>& pos, const std::string& loc)
+void Avatar::drop(Entity* e, const WFMath::Point<3>& pos, const WFMath::Quaternion& orientation, const std::string& loc)
 {
     if(e->getLocation() != m_entity)
     {
@@ -86,15 +92,18 @@ void Avatar::drop(Entity* e, const WFMath::Point<3>& pos, const std::string& loc
     what->setLoc(loc);
     Atlas::Message::Element apos(pos.toAtlas());
     what->setPosAsList(apos.asList());
+    if (orientation.isValid()) {
+        what->setAttr("orientation", orientation.toAtlas());
+    }
     what->setId(e->getId());
     moveOp->setArgs1(what);
 
     getConnection()->send(moveOp);
 }
 
-void Avatar::drop(Entity* e, const WFMath::Vector<3>& offset)
+void Avatar::drop(Entity* e, const WFMath::Vector<3>& offset, const WFMath::Quaternion& orientation)
 {
-    drop(e, m_entity->getPosition() + offset, m_entity->getLocation()->getId());
+    drop(e, m_entity->getPosition() + offset, orientation, m_entity->getLocation()->getId());
 }
 
 void Avatar::take(Entity* e)
@@ -138,6 +147,24 @@ void Avatar::say(const std::string& msg)
     getConnection()->send(t);
 }
 
+void Avatar::sayTo(const std::string& message, const std::vector<const Entity*>& entities)
+{
+    Talk t;
+
+    Anonymous what;
+    what->setAttr("say", message);
+    Atlas::Message::ListType addressList;
+    for (std::vector<const Entity*>::const_iterator I = entities.begin(); I != entities.end(); ++I) {
+        addressList.push_back((*I)->getId());
+    }
+    what->setAttr("address", addressList);
+    t->setArgs1(what);
+    t->setFrom(m_entityId);
+
+    getConnection()->send(t);
+}
+
+
 void Avatar::emote(const std::string &em)
 {
     Imaginary im;
@@ -176,19 +203,20 @@ void Avatar::moveInDirection(const WFMath::Vector<3>& vel)
     arg->setId(m_entityId);
     arg->setAttr("velocity", vel.toAtlas());
 
-    WFMath::CoordType sqr_mag = vel.sqrMag();
+    CoordType sqr_mag = vel.sqrMag();
     if(sqr_mag > MIN_VELOCITY) { // don't set orientation for zero velocity
         WFMath::Quaternion q;
-        WFMath::CoordType z_squared = vel.z() * vel.z();
-        WFMath::CoordType plane_sqr_mag = sqr_mag - z_squared;
-        if(plane_sqr_mag < WFMATH_EPSILON * z_squared) {
+        CoordType z_squared = vel.z() * vel.z();
+        CoordType plane_sqr_mag = sqr_mag - z_squared;
+        if(plane_sqr_mag < numeric_constants<CoordType>::epsilon() * z_squared) {
             // it's on the z axis
-            q.rotation(1, vel[2] > 0 ? -WFMath::Pi/2 : WFMath::Pi/2);
+            q.rotation(1, vel[2] > 0 ? -numeric_constants<CoordType>::pi()/2
+                                     : numeric_constants<CoordType>::pi()/2);
         } else {
             // rotate in the plane first
-            q.rotation(2, atan2(vel[1], vel[0]));
+            q.rotation(2, std::atan2(vel[1], vel[0]));
             // then get the angle away from the plane
-            q = WFMath::Quaternion(1, -asin(vel[2] / sqrt(plane_sqr_mag))) * q;
+            q = WFMath::Quaternion(1, -std::asin(vel[2] / std::sqrt(plane_sqr_mag))) * q;
         }
 
         arg->setAttr("orientation", q.toAtlas());
@@ -216,19 +244,35 @@ void Avatar::moveInDirection(const WFMath::Vector<3>& vel, const WFMath::Quatern
     getConnection()->send(moveOp);
 }
 
-void Avatar::place(Entity* e, Entity* container, const WFMath::Point<3>& pos)
+void Avatar::place(Entity* entity, Entity* container, const WFMath::Point<3>& pos,
+        const WFMath::Quaternion& orientation)
 {
     Anonymous what;
     what->setLoc(container->getId());
-    what->setAttr("pos", pos.toAtlas());
-   // what->setVelocityAsList( .... zero ... );
-    what->setId(e->getId());
+    if (pos.isValid()) {
+        what->setPosAsList(Atlas::Message::Element(pos.toAtlas()).asList());
+    } else {
+        what->setAttr("pos", WFMath::Point<3>::ZERO().toAtlas());
+    }
+    if (orientation.isValid()) {
+        what->setAttr("orientation", orientation.toAtlas());
+    }
+
+    what->setId(entity->getId());
 
     Move moveOp;
     moveOp->setFrom(m_entityId);
     moveOp->setArgs1(what);
 
+    //if the avatar is an admin, we will set the TO property
+    //this will bypass all of the server's filtering, allowing us to place any
+    //entity, unrelated to if it's too heavy or belong to someone else
+    if (getIsAdmin()) {
+        moveOp->setTo(entity->getId());
+    }
+
     getConnection()->send(moveOp);
+
 }
 
 void Avatar::wield(Entity * entity)
@@ -337,9 +381,14 @@ void Avatar::onCharacterWield(const Atlas::Message::Element& val)
     m_wielded = EntityRef(m_view, val.asString());
 }
 
+void Avatar::onTransferRequested(const TransferInfo &transfer)
+{
+    TransferRequested.emit(transfer);
+}
+
 Connection* Avatar::getConnection() const
 {
-    return m_account->getConnection();
+    return m_account.getConnection();
 }
 
 double Avatar::getWorldTime()
@@ -383,9 +432,30 @@ void Avatar::logoutResponse(const RootOperation& op)
         return;
     }
 
-    m_account->AvatarDeactivated.emit(this);
+    m_account.AvatarDeactivated.emit(this);
     deleteLater(this);
 }
 
+void Avatar::logoutRequested()
+{
+    m_account.avatarLogoutRequested(this);
+}
+
+void Avatar::logoutRequested(const TransferInfo& transferInfo)
+{
+    onTransferRequested(transferInfo);
+    m_account.avatarLogoutRequested(this);
+}
+
+void Avatar::setIsAdmin(bool isAdmin)
+{
+    m_isAdmin = isAdmin;
+}
+
+bool Avatar::getIsAdmin()
+{
+    return m_isAdmin;
+}
+
 
 } // of namespace Eris
