@@ -6,8 +6,6 @@
 #include <Eris/Connection.h>
 #include <Eris/TypeInfo.h>
 #include <Eris/LogStream.h>
-#include <Eris/EntityRouter.h>
-#include <Eris/View.h>
 #include <Eris/Exceptions.h>
 #include <Eris/Avatar.h>
 #include <Eris/Alarm.h>
@@ -23,6 +21,8 @@
 #include <cmath>
 #include <cassert>
 
+#pragma warning(disable: 4068)  //unknown pragma
+
 using namespace Atlas::Objects::Operation;
 using Atlas::Objects::Root;
 using Atlas::Objects::Entity::RootEntity;
@@ -37,7 +37,7 @@ using WFMath::TimeDiff;
 
 namespace Eris {
 
-Entity::Entity(const std::string& id, TypeInfo* ty, View* vw) :
+Entity::Entity(const std::string& id, TypeInfo* ty) :
     m_type(ty),
     m_location(NULL),
     m_id(id),
@@ -45,7 +45,6 @@ Entity::Entity(const std::string& id, TypeInfo* ty, View* vw) :
     m_visible(false),
     m_limbo(false),
     m_updateLevel(0),
-    m_view(vw),
     m_hasBBox(false),
     m_moving(false),
     m_recentlyCreated(false),
@@ -54,8 +53,6 @@ Entity::Entity(const std::string& id, TypeInfo* ty, View* vw) :
     assert(m_id.size() > 0);
     m_orientation.identity();
     
-    m_router = new EntityRouter(this);
-    m_view->getConnection()->registerRouterForFrom(m_router, m_id);
     
     if (m_type) {
         m_type->AttributeChanges.connect(sigc::mem_fun(this, &Entity::typeInfo_AttributeChanges));
@@ -70,7 +67,16 @@ Entity::~Entity()
 void Entity::shutdown()
 {
     BeingDeleted.emit();
-    if (m_moving) m_view->removeFromPrediction(this);
+
+    //Delete any lingering tasks.
+    for (TaskArray::const_iterator it = m_tasks.begin(); it != m_tasks.end(); ++it) {
+        TaskRemoved(*it);
+        delete *it;
+    }
+
+    if (m_moving) {
+    	removeFromMovementPrediction();
+    }
     
     while (!m_contents.empty()) {
       Entity *e = m_contents.back();
@@ -79,9 +85,6 @@ void Entity::shutdown()
     }
     setLocation(NULL);
     
-    m_view->getConnection()->unregisterRouterForFrom(m_router, m_id);
-    m_view->entityDeleted(this); // remove ourselves from the View's content map
-    delete m_router;
 
     m_initialised = false;
 }
@@ -154,7 +157,7 @@ void Entity::fillAttributesFromType(Entity::AttrMap& attributes, TypeInfo* typeI
 {
     attributes.insert(typeInfo->getAttributes().begin(), typeInfo->getAttributes().end());
     ///Make sure to fill from the closest attributes first, as insert won't replace an existing value
-    for (TypeInfoSet::iterator I = typeInfo->getParents().begin(); I != typeInfo->getParents().end(); ++I) {
+    for (TypeInfoSet::const_iterator I = typeInfo->getParents().begin(); I != typeInfo->getParents().end(); ++I) {
         fillAttributesFromType(attributes, *I);
     }
 }
@@ -168,8 +171,10 @@ sigc::connection Entity::observe(const std::string& attr, const AttrChangedSlot&
 WFMath::Point<3> Entity::getViewPosition() const
 {
     WFMath::Point<3> vpos(0.0, 0.0, 0.0);
+    //If the position is invalid, we will consider it to be (0,0,0) and skip applying it.
     for (const Entity* e = this; e; e = e->getLocation())
-        vpos = e->toLocationCoords(vpos);
+        if (e->getPosition().isValid())
+            vpos = e->toLocationCoords(vpos);
         
     return vpos;
 }
@@ -209,11 +214,11 @@ void Entity::updatePredictedState(const WFMath::TimeStamp& t)
 {
     assert(isMoving());
     
-    double dt = (t - m_lastMoveTime).milliseconds() / 1000.0; 
+    float dt = (t - m_lastMoveTime).milliseconds() / 1000.0f; 
     
     if (m_acc.isValid()) {
         m_predicted.velocity = m_velocity + (m_acc * dt);
-        m_predicted.position = m_position + (m_velocity * dt) + (m_acc * 0.5 * dt * dt);
+        m_predicted.position = m_position + (m_velocity * dt) + (m_acc * 0.5f * dt * dt);
     } else {
         m_predicted.velocity = m_velocity;
         m_predicted.position = m_position + (m_velocity * dt);
@@ -233,7 +238,7 @@ TypeInfoArray Entity::getUseOperations() const
     const ListType& opsl(it->second.asList());
     TypeInfoArray useOps;
     useOps.reserve(opsl.size());
-    TypeService* ts = m_view->getAvatar()->getConnection()->getTypeService();
+    TypeService* ts = getTypeService();
     
     for (ListType::const_iterator i=opsl.begin(); i!=opsl.end(); ++i)
     {
@@ -265,7 +270,7 @@ void Entity::setFromRoot(const Root& obj, bool allowMove, bool includeTypeInfoAt
     
     Atlas::Message::MapType attrs;
     obj->addToMessage(attrs);
-    Atlas::Message::MapType::iterator A;
+    Atlas::Message::MapType::const_iterator A;
     
     ///Fill with the default values from the type info
     if (includeTypeInfoAttributes && m_type) {
@@ -280,7 +285,7 @@ void Entity::setFromRoot(const Root& obj, bool allowMove, bool includeTypeInfoAt
     
     for (A = attrs.begin(); A != attrs.end(); ++A) {
         // see if the value in the sight matches the exsiting value
-        AttrMap::iterator I = m_attrs.find(A->first);
+        AttrMap::const_iterator I = m_attrs.find(A->first);
         if ((I != m_attrs.end()) && (I->second == A->second)) continue;
 
         setAttr(A->first, A->second);
@@ -309,7 +314,6 @@ void Entity::onTalk(const Atlas::Objects::Operation::RootOperation& talk)
 
     Say.emit(talkArgs.front());
     Noise.emit(talk);
-    m_view->getAvatar()->Hear.emit(this, talk);
 }
 
 void Entity::onLocationChanged(Entity* oldLoc)
@@ -330,7 +334,6 @@ void Entity::onAction(const Atlas::Objects::Operation::RootOperation& arg)
 void Entity::onSoundAction(const Atlas::Objects::Operation::RootOperation& op)
 {
     Noise.emit(op);
-    m_view->getAvatar()->Hear.emit(this, op);
 }
 
 void Entity::onImaginary(const Atlas::Objects::Root& arg)
@@ -344,12 +347,14 @@ void Entity::setMoving(bool inMotion)
 {
     assert(m_moving != inMotion);
     
-    if (m_moving) m_view->removeFromPrediction(this);
+    if (m_moving) {
+    	removeFromMovementPrediction();
+    }
     m_moving = inMotion;
     if (m_moving) {
         m_predicted.position = m_position;
         m_predicted.velocity = m_velocity;
-        m_view->addToPrediction(this);
+        addToMovementPredition();
     }
     
     Moving.emit(inMotion);
@@ -365,6 +370,12 @@ void Entity::onChildRemoved(Entity* child)
     ChildRemoved(child);
 }
 
+void Entity::onTaskAdded(Task* task)
+{
+	TaskAdded(task);
+}
+
+
 #pragma mark -
 
 void Entity::setAttr(const std::string &attr, const Element &val)
@@ -374,7 +385,7 @@ void Entity::setAttr(const std::string &attr, const Element &val)
     
     ///Check whether the attribute already has been added to the instance attributes.
     const Element* typeElement(0);
-    AttrMap::iterator A = m_attrs.find(attr);
+    AttrMap::const_iterator A = m_attrs.find(attr);
     if (A == m_attrs.end() && m_type) {
         ///If the attribute hasn't been defined for this instance, see if there's a typeinfo default one
         typeElement = m_type->getAttribute(attr);
@@ -384,7 +395,7 @@ void Entity::setAttr(const std::string &attr, const Element &val)
     const Element* newElement(0);
     ///There was no preexisting attribute either in this instance or in the TypeInfo, just insert it.
     if (A == m_attrs.end() && typeElement == 0) {
-        std::pair<AttrMap::iterator, bool> I = m_attrs.insert(AttrMap::value_type(attr, val));
+        std::pair<AttrMap::const_iterator, bool> I = m_attrs.insert(AttrMap::value_type(attr, val));
         newElement = &(I.first->second);
     } else {
         ///Create an instance specific attribute
@@ -555,9 +566,9 @@ void Entity::updateTasks(const Element& e)
     TaskArray previousTasks(m_tasks);
     m_tasks.clear();
     
-    for (unsigned int t=0; t<taskList.size(); ++t)
+    for (unsigned int i=0; i<taskList.size(); ++i)
     {
-        const MapType& tkmap(taskList[t].asMap());
+        const MapType& tkmap(taskList[i].asMap());
         MapType::const_iterator it = tkmap.find("name");
         if (it == tkmap.end())
         {
@@ -566,19 +577,19 @@ void Entity::updateTasks(const Element& e)
         }
         
         int index = findTaskByName(previousTasks, it->second.asString());
-        Task *t;
+        Task *task;
         
         if (index < 0)
         {   // not found, create a new one
-            t = new Task(this, it->second.asString());
-            TaskAdded.emit(t);
+            task = new Task(this, it->second.asString());
+            onTaskAdded(task);
         } else {
-            t = previousTasks[index];
+            task = previousTasks[index];
             previousTasks[index] = NULL;
         }
         
-        m_tasks.push_back(t);
-        t->updateFromAtlas(tkmap);
+        m_tasks.push_back(task);
+        task->updateFromAtlas(tkmap);
     } // of Atlas-specified tasks iteration
     
     for (unsigned int d=0; d<previousTasks.size(); ++d)
@@ -605,25 +616,26 @@ void Entity::removeTask(Task* t)
 
 #pragma mark -
 
-void Entity::setLocationFromAtlas(const std::string& locId)
-{
-    if (locId.empty()) return;
-    
-    Entity* newLocation = m_view->getEntity(locId);
-    if (!newLocation)
-    {
-        m_view->getEntityFromServer(locId);
-        
-        m_limbo = true;
-        setVisible(false); // fire disappearance, VisChanged if necessary
-        
-        if (m_location) removeFromLocation();
-        m_location = NULL;
-        assert(m_visible == false);
-        return;
-    }
-    
-    setLocation(newLocation);
+void Entity::setLocationFromAtlas(const std::string& locId) {
+	if (locId.empty()) {
+		return;
+	}
+
+	Entity* newLocation = getEntity(locId);
+	if (!newLocation) {
+
+		m_limbo = true;
+		setVisible(false); // fire disappearance, VisChanged if necessary
+
+		if (m_location) {
+			removeFromLocation();
+		}
+		m_location = NULL;
+		assert(m_visible == false);
+		return;
+	}
+
+	setLocation(newLocation);
 }
 
 void Entity::setLocation(Entity* newLocation)
@@ -632,7 +644,9 @@ void Entity::setLocation(Entity* newLocation)
         
 // do the actual member updating
     bool wasVisible = isVisible();
-    if (m_location) removeFromLocation();
+    if (m_location) {
+    	removeFromLocation();
+    }
         
     Entity* oldLocation = m_location;
     m_location = newLocation;
@@ -642,7 +656,9 @@ void Entity::setLocation(Entity* newLocation)
 // fire VisChanged and Appearance/Disappearance signals
     updateCalculatedVisibility(wasVisible);
     
-    if (m_location) addToLocation();    
+    if (m_location) {
+    	addToLocation();
+    }
 }
 
 void Entity::addToLocation()
@@ -683,13 +699,9 @@ void Entity::setContentsFromAtlas(const StringList& contents)
             assert(child->getLocation() == this);
             oldContents.erase(J);
         } else {
-            child = m_view->getEntity(*I);
+            child = getEntity(*I);
             if (!child) {
-                // we don't have the entity at all, so request it and skip
-                // processing it here; everything will come right when it
-                // arrives.
-                m_view->getEntityFromServer(*I);
-                continue;
+            	continue;
             }
             
             if (child->m_limbo) {
@@ -783,7 +795,6 @@ void Entity::updateCalculatedVisibility(bool wasVisible)
     Appearances top-down, but Disappearances bottom-up. */
     
     if (nowVisible) {
-        m_view->setEntityVisible(this, true);
         onVisibilityChanged(true);
     }
     
@@ -797,7 +808,6 @@ void Entity::updateCalculatedVisibility(bool wasVisible)
     }
     
     if (wasVisible) {
-        m_view->setEntityVisible(this, false);
         onVisibilityChanged(false);
     }
 }
